@@ -1,4 +1,4 @@
-/* $Id: actions.cc 833 2008-03-13 17:36:01Z helly $ */
+/* $Id: actions.cc 850 2008-03-24 01:05:08Z helly $ */
 #include <time.h>
 #include <string.h>
 #include <iostream>
@@ -854,12 +854,24 @@ RegExp * Scanner::ranToRE(SubStr s) const
 	return new MatchOp(r);
 }
 
+RegExp * Scanner::getAnyRE() const
+{
+	if (eFlag)
+	{
+		return ranToRE(SubStr("[\\000-\\377]"));
+	}
+	else
+	{
+		return new MatchOp(new Range(0, nRealChars));
+	}
+}
+
 RegExp * Scanner::invToRE(SubStr s) const
 {
 	s.len--;
 	s.str++;
 	
-	RegExp * any = ranToRE(SubStr(wFlag ? "[\\X0000-\\XFFFF]" : "[\\000-\\377]"));
+	RegExp * any = getAnyRE();
 
 	if (s.len <= 2)
 	{
@@ -877,7 +889,7 @@ RegExp * Scanner::invToRE(SubStr s) const
 
 RegExp * Scanner::mkDot() const
 {
-	RegExp * any = ranToRE(SubStr(wFlag ? "[\\X0000-\\XFFFF]" : "[\\000-\\377]"));
+	RegExp * any = getAnyRE();
 	RegExp * ran = matchChar(xlat('\n'));
 	RegExp * inv = mkDiff(any, ran);
 	
@@ -1005,7 +1017,7 @@ CharSet::~CharSet()
 	delete[] ptn;
 }
 
-DFA* genCode(std::ostream& o, uint& ind, RegExp *re, const RegExpMap* specMap, const std::string& condName, bool isLastCond, bool &bPrologBrace)
+DFA* genCode(RegExp *re)
 {
 	CharSet cs;
 	uint j;
@@ -1052,9 +1064,7 @@ DFA* genCode(std::ostream& o, uint& ind, RegExp *re, const RegExpMap* specMap, c
 		}
 	}
 
-	DFA *dfa = new DFA(ins, re->size, 0, nRealChars, rep);
-	dfa->emit(o, ind, specMap, condName, isLastCond, bPrologBrace);
-	return dfa;
+	return new DFA(ins, re->size, 0, nRealChars, rep);
 }
 
 } // end namespace re2c
