@@ -1,4 +1,5 @@
-/* $Id: code.cc 834 2008-03-13 17:38:12Z helly $ */
+/* $Id: code.cc 861 2008-05-25 13:23:02Z helly $ */
+#include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -420,14 +421,21 @@ static void need(std::ostream &o, uint ind, uint n, bool & readCh, bool bSetMark
 
 	if (bUseYYFill && n > 0)
 	{
+		o << indent(ind);
 		if (n == 1)
 		{
-			o << indent(ind) << "if (" << mapCodeName["YYLIMIT"] << " == " << mapCodeName["YYCURSOR"] << ") ";
+			if (bUseYYFillCheck)
+			{
+				o << "if (" << mapCodeName["YYLIMIT"] << " <= " << mapCodeName["YYCURSOR"] << ") ";
+			}
 			genYYFill(o, ind, n);
 		}
 		else
 		{
-			o << indent(ind) << "if ((" << mapCodeName["YYLIMIT"] << " - " << mapCodeName["YYCURSOR"] << ") < " << n << ") ";
+			if (bUseYYFillCheck)
+			{
+				o << "if ((" << mapCodeName["YYLIMIT"] << " - " << mapCodeName["YYCURSOR"] << ") < " << n << ") ";
+			}
 			genYYFill(o, ind, n);
 		}
 	}
@@ -728,7 +736,7 @@ void Rule::emit(std::ostream &o, uint ind, bool &, const std::string& condName)
 		o << indent(ind) << yySetupRule << "\n";
 	}
 
-	o << file_info(sourceFileInfo, &rl);
+	o << file_info(rule->code->source, &rl);
 	o << indent(ind);
 	if (rule->code->autogen)
 	{
@@ -1548,7 +1556,7 @@ void DFA::findBaseState()
 	delete [] span;
 }
 
-void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::string& condName, bool isLastCond, bool& bPrologBrace)
+void DFA::prepare()
 {
 	State *s;
 	uint i;
@@ -1559,7 +1567,6 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 	head->link = head;
 
 	uint nRules = 0;
-	bool bProlog = (!cFlag || !bWroteCondCheck);
 
 	for (s = head; s; s = s->next)
 	{
@@ -1575,11 +1582,11 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 	}
 
 	uint nSaves = 0;
-	uint *saves = new uint[nRules];
+	saves = new uint[nRules];
 	memset(saves, ~0, (nRules)*sizeof(*saves));
 
 	// mark backtracking points
-	bool bSaveOnHead = false;
+	bSaveOnHead = false;
 
 	for (s = head; s; s = s->next)
 	{
@@ -1605,7 +1612,7 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 	}
 
 	// insert actions
-	State **rules = new State * [nRules];
+	rules = new State * [nRules];
 
 	memset(rules, 0, (nRules)*sizeof(*rules));
 
@@ -1685,6 +1692,12 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 
 	delete head->action;
 	head->action = NULL;
+}
+
+
+void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::string& condName, bool isLastCond, bool& bPrologBrace)
+{
+	bool bProlog = (!cFlag || !bWroteCondCheck);
 
 	if (!cFlag)
 	{
@@ -1703,6 +1716,8 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 		}
 	}
 
+	State *s;
+
 	for (s = head; s; s = s->next)
 	{
 		s->label = next_label++;
@@ -1842,9 +1857,6 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 		BitMap::first = NULL;
 	}
 
-	delete [] saves;
-	delete [] rules;
-
 	bUseStartLabel = false;
 }
 
@@ -2075,11 +2087,6 @@ std::ostream& operator << (std::ostream& o, const file_info& li)
 	return o;
 }
 
-uint Scanner::get_line() const
-{
-	return cline;
-}
-
 void Scanner::config(const Str& cfg, int num)
 {
 	if (cfg.to_string() == "indent:top")
@@ -2115,6 +2122,10 @@ void Scanner::config(const Str& cfg, int num)
 	{
 		bUseYYFillParam = num != 0;
 	}
+	else if (cfg.to_string() == "yyfill:check")
+	{
+		bUseYYFillCheck = num != 0;
+	}
 	else if (cfg.to_string() == "cgoto:threshold")
 	{
 		cGotoThreshold = num;
@@ -2148,12 +2159,29 @@ void Scanner::config(const Str& cfg, int num)
 	{
 		bUseYYGetStateNaked = num != 0;
 	}
+	else if (cfg.to_string() == "define:YYSETSTATE:naked")
+	{
+		bUseYYSetStateNaked = num != 0;
+	}
+	else if (cfg.to_string() == "flags:u")
+	{
+		if (!rFlag)
+		{
+			fatalf("cannot use configuration name '%s' without -r flag", cfg.to_string().c_str());
+		}
+		uFlag = num != 0;
+	}
+	else if (cfg.to_string() == "flags:w")
+	{
+		if (!rFlag)
+		{
+			fatalf("cannot use configuration name '%s' without -r flag", cfg.to_string().c_str());
+		}
+		wFlag = num != 0;
+	}
 	else
 	{
-		std::string msg = "unrecognized configuration name '";
-		msg += cfg.to_string();
-		msg += "' or illegal integer value";
-		fatal(msg.c_str());
+		fatalf("unrecognized configuration name '%s' or illegal integer value", cfg.to_string().c_str());
 	}
 }
 
@@ -2288,4 +2316,176 @@ void Scanner::config(const Str& cfg, const Str& val)
 	}
 }
 
+ScannerState::ScannerState()
+	: tok(NULL), ptr(NULL), cur(NULL), pos(NULL), ctx(NULL)
+	, bot(NULL), lim(NULL), top(NULL), eof(NULL)
+	, tchar(0), tline(0), cline(1), iscfg(0)
+	, in_parse(false)
+{
+}
+
+Scanner::Scanner(std::istream& i, std::ostream& o)
+	: ScannerState(), in(i), out(o)
+{
+}
+
+char *Scanner::fill(char *cursor, uint need)
+{
+	if(!eof)
+	{
+		uint cnt;
+		/* Do not get rid of anything when rFlag is active. Otherwise
+		 * get rid of everything that was already handedout. */
+		if (!rFlag)
+		{
+			cnt = tok - bot;
+			if (cnt)
+			{
+				memmove(bot, tok, top - tok);
+				tok  = bot;
+				ptr -= cnt;
+				cur -= cnt;
+				pos -= cnt;
+				lim -= cnt;
+				ctx -= cnt;
+				cursor -= cnt;
+			}
+		}
+		/* In crease buffer size. */
+		if (BSIZE > need)
+		{
+			need = BSIZE;
+		}
+		if (static_cast<uint>(top - lim) < need)
+		{
+			char *buf = new char[(lim - bot) + need];
+			if (!buf)
+			{
+				fatal("Out of memory");
+			}
+			memcpy(buf, bot, lim - bot);
+			tok = &buf[tok - bot];
+			ptr = &buf[ptr - bot];
+			cur = &buf[cur - bot];
+			pos = &buf[pos - bot];
+			lim = &buf[lim - bot];
+			top = &lim[need];
+			ctx = &buf[ctx - bot];
+			cursor = &buf[cursor - bot];
+			delete [] bot;
+			bot = buf;
+		}
+		/* Append to buffer. */
+		in.read(lim, need);
+		if ((cnt = in.gcount()) != need)
+		{
+			eof = &lim[cnt];
+			*eof++ = '\0';
+		}
+		lim += cnt;
+	}
+	return cursor;
+}
+
+void Scanner::set_in_parse(bool new_in_parse)
+{
+	in_parse = new_in_parse;
+}
+
+void Scanner::fatal_at(uint line, uint ofs, const char *msg) const
+{
+	out.flush();
+	std::cerr << "re2c: error: "
+		<< "line " << line << ", column " << (tchar + ofs + 1) << ": "
+		<< msg << std::endl;
+	exit(1);
+}
+
+void Scanner::fatal(uint ofs, const char *msg) const
+{
+	fatal_at(in_parse ? tline : cline, ofs, msg);
+}
+
+void Scanner::fatalf_at(uint line, const char* fmt, ...) const
+{
+	char szBuf[4096];
+
+	va_list args;
+	
+	va_start(args, fmt);
+	vsnprintf(szBuf, sizeof(szBuf), fmt, args);
+	va_end(args);
+	
+	szBuf[sizeof(szBuf)-1] = '0';
+	
+	fatal_at(line, 0, szBuf);
+}
+
+void Scanner::fatalf(const char *fmt, ...) const
+{
+	char szBuf[4096];
+
+	va_list args;
+	
+	va_start(args, fmt);
+	vsnprintf(szBuf, sizeof(szBuf), fmt, args);
+	va_end(args);
+	
+	szBuf[sizeof(szBuf)-1] = '0';
+	
+	fatal(szBuf);
+}
+
+Scanner::~Scanner()
+{
+	if (bot)
+	{
+		delete [] bot;
+	}
+}
+
+void Scanner::check_token_length(char *pos, uint len) const
+{
+	if (pos < bot || pos + len > top)
+	{
+		fatal("Token exceeds limit");
+	}
+}
+
+Str Scanner::raw_token(std::string enclosure) const
+{
+	return Str(std::string(enclosure + token().to_string() + enclosure).c_str());
+}
+
+void Scanner::reuse()
+{
+	next_label = 0;
+	next_fill_index = 0;
+	bWroteGetState = false;
+	bWroteCondCheck = false;
+	mapCodeName.clear();
+}
+
+void Scanner::restore_state(const ScannerState& state)
+{
+	int diff = bot - state.bot;
+	char *old_bot = bot;
+	char *old_lim = lim;
+	char *old_top = top;
+	char *old_eof = eof;
+	*(ScannerState*)this = state;
+	if (diff)
+	{
+		tok -= diff;
+		ptr -= diff;
+		cur -= diff;
+		pos -= diff;
+		ctx -= diff;		
+		bot = old_bot;
+		lim = old_lim;
+		top = old_top;
+		eof = old_eof;
+	}
+}
+
 } // end namespace re2c
