@@ -1,4 +1,4 @@
-/* $Id: dfa.h 829 2008-03-07 21:59:19Z helly $ */
+/* $Id: dfa.h 850 2008-03-24 01:05:08Z helly $ */
 #ifndef _dfa_h
 #define _dfa_h
 
@@ -259,6 +259,11 @@ public:
 	const Ins     *free_ins;
 	const Char    *free_rep;
 
+protected:
+	bool    bSaveOnHead;
+	uint    *saves;
+	State   **rules;
+
 public:
 	DFA(Ins*, uint, uint, uint, const Char*);
 	~DFA();
@@ -268,6 +273,7 @@ public:
 
 	void findSCCs();
 	void findBaseState();
+	void prepare();
 	void emit(std::ostream&, uint&, const RegExpMap*, const std::string&, bool, bool&);
 
 	friend std::ostream& operator<<(std::ostream&, const DFA&);
