@@ -1,4 +1,4 @@
-/* $Id: globals.h 829 2008-03-07 21:59:19Z helly $ */
+/* $Id: globals.h 855 2008-03-24 18:46:49Z helly $ */
 #ifndef	_globals_h
 #define	_globals_h
 
@@ -12,6 +12,8 @@
 namespace re2c
 {
 
+enum { BSIZE = 8192};
+
 extern file_info sourceFileInfo;
 extern file_info outputFileInfo;
 extern file_info headerFileInfo;
@@ -69,10 +71,12 @@ extern bool bUseStateAbort;
 extern bool bUseStateNext;
 extern bool bUseYYFill;
 extern bool bUseYYFillParam;
+extern bool bUseYYFillCheck;
 extern bool bUseYYFillNaked;
 extern bool bUseYYSetConditionParam;
 extern bool bUseYYGetConditionNaked;
 extern bool bUseYYSetStateParam;
+extern bool bUseYYSetStateNaked;
 extern bool bUseYYGetStateNaked;
 extern bool bWroteGetState;
 extern bool bWroteCondCheck;
