@@ -1,4 +1,4 @@
-/* $Id: main.cc 829 2008-03-07 21:59:19Z helly $ */
+/* $Id: main.cc 858 2008-04-03 20:53:44Z helly $ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #elif defined(_WIN32)
@@ -53,10 +53,12 @@ bool bUseStartLabel  = false;
 bool bUseStateNext   = false;
 bool bUseYYFill      = true;
 bool bUseYYFillParam = true;
+bool bUseYYFillCheck = true;
 bool bUseYYFillNaked = false;
 bool bUseYYSetConditionParam = true;
 bool bUseYYGetConditionNaked = false;
 bool bUseYYSetStateParam = true;
+bool bUseYYSetStateNaked = false;
 bool bUseYYGetStateNaked = false;
 
 std::string startLabelName;
@@ -132,7 +134,7 @@ static const mbo_opt_struct OPTIONS[] =
 
 static void usage()
 {
-	cerr << "usage: re2c [-bcdDefFghisvVw1] [-o file] file\n"
+	std::cerr << "usage: re2c [-bcdDefFghirsuvVw1] [-o of] [-t th] file\n"
 	"\n"
 	"-? -h  --help           Display this info.\n"
 	"\n"
@@ -149,8 +151,8 @@ static void usage()
 	"\n"
 	"-D     --emit-dot       Emit a Graphviz dot view of the DFA graph\n"
 	"\n"
-	"-e     --ecb            Cross-compile from an ASCII platform to\n"
-	"                        an EBCDIC one.\n"
+	"-e     --ecb            Cross-compile from an ASCII platform to an EBCDIC one.\n"
+	"                        This cannot be combined with -w, -u or -r.\n"
 	"\n"
 	"-f     --storable-state Generate a scanner that supports storable states.\n"
 	"\n"
@@ -161,17 +163,18 @@ static void usage()
 	"\n"
 	"-i     --no-debug-info  Do not generate '#line' info (usefull for versioning).\n"
 	"\n"
-	"-o     --output=output  Specify the output file instead of stdout\n"
-	"                        This cannot be used together with -e switch.\n"
+	"-o of  --output=of      Specify the output file (of) instead of stdout\n"
 	"\n"
 	"-r     --reusable       Allow reuse of scanner definitions.\n"
+	"                        This cannot be used together with -e switch.\n"
 	"\n"
 	"-s     --nested-ifs     Generate nested ifs for some switches. Many compilers\n"
 	"                        need this assist to generate better code.\n"
 	"\n"
-	"-t     --type-header    Generate a header file with type definitions.\n"
+	"-t th  --type-header=th Generate a type header file (th) with type definitions.\n"
 	"\n"
 	"-u     --unicode        Implies -w but supports the full Unicode character set.\n"
+	"                        This cannot be used together with -e switch.\n"
 	"\n"
 	"-v     --version        Show version information.\n"
 	"\n"
@@ -213,7 +216,7 @@ int main(int argc, char *argv[])
 		return 2;
 	}
 
-	while ((c = mbo_getopt(argc, argv, OPTIONS, &opt_arg, &opt_ind, 0)) != -1)
+	while ((c = mbo_getopt(argc, argv, OPTIONS, &opt_arg, &opt_ind, 1)) != -1)
 	{
 		switch (c)
 		{
@@ -319,9 +322,9 @@ int main(int argc, char *argv[])
 			uFlag = true;
 			break;
 	  
+			default:
 			case 'h':
 			case '?':
-			default:
 			usage();
 			return 2;
 
@@ -349,6 +352,11 @@ int main(int argc, char *argv[])
 		return 2;
 	}
 
+	if (rFlag && eFlag)
+	{
+		std::cerr << "re2c: error: Cannot combine -e with -r switch\n";
+		return 2;
+	}
 	if (wFlag && eFlag)
 	{
 		std::cerr << "re2c: error: Cannot combine -e with -w or -u switch\n";
@@ -377,6 +385,7 @@ int main(int argc, char *argv[])
 	}
 	else
 	{
+		std::cerr << "Unexpected argument: " << argv[argc-1] << "\n\n";
 		usage();
 		return 2;
 	}
@@ -448,6 +457,7 @@ int main(int argc, char *argv[])
 		bWroteCondCheck = false;
 		bUsedYYMaxFill = false;
 		bFirstPass = false;
+		sourceFileInfo.set_fname(sourceFileName);
 	}
 
 	bLastPass = true;
