@@ -1,4 +1,4 @@
-/* $Id: re.h 836 2008-03-13 17:44:34Z helly $ */
+/* $Id: re.h 850 2008-03-24 01:05:08Z helly $ */
 #ifndef _re_h
 #define _re_h
 
@@ -501,7 +501,7 @@ typedef std::map<std::string, LineCode> SetupMap;
 
 class DFA;
 
-extern DFA* genCode(std::ostream&, uint&, RegExp*, const RegExpMap*, const std::string&, bool, bool&);
+extern DFA* genCode(RegExp*);
 extern void genGetStateGoto(std::ostream&, uint&, uint);
 extern void genCondTable(std::ostream&, uint, const RegExpMap&);
 extern void genCondGoto(std::ostream&, uint, const RegExpMap&);
