@@ -1,4 +1,4 @@
-/* $Id: scanner.h 835 2008-03-13 17:43:07Z helly $ */
+/* $Id: scanner.h 860 2008-04-09 22:57:45Z nuno-lopes $ */
 #ifndef _scanner_h
 #define	_scanner_h
 
@@ -11,20 +11,28 @@
 namespace re2c
 {
 
+struct ScannerState
+{
+	ScannerState();
+
+	char	*tok, *ptr, *cur, *pos, *ctx;  // positioning
+	char    *bot, *lim, *top, *eof;        // buffer
+	uint	tchar, tline, cline, iscfg, buf_size;
+	bool    in_parse;
+};
+
 class Scanner:
-	public line_number
+	public line_number, private ScannerState
 {
 private:
 	std::istream&	in;
 	std::ostream&   out;
-	char	*bot, *tok, *ptr, *cur, *pos, *lim, *top, *eof, *ctx;
-	uint	tchar, tline, cline, iscfg, buf_size;
-	bool    in_parse;
 
 private:
 	char *fill(char*, uint);
 	Scanner(const Scanner&); //unimplemented
 	Scanner& operator=(const Scanner&); //unimplemented
+	void set_sourceline(char *& cursor);
 
 public:
 	Scanner(std::istream&, std::ostream&);
@@ -33,11 +41,17 @@ public:
 	enum ParseMode {
 		Stop,
 		Parse,
-		Reuse
+		Reuse,
+		Rules
 	};
 
 	ParseMode echo();
 	int scan();
+	void reuse();
+	
+	size_t get_pos() const;
+	void save_state(ScannerState&) const;
+	void restore_state(const ScannerState&);
 
 	uint get_cline() const;
 	void set_in_parse(bool new_in_parse);
@@ -53,7 +67,7 @@ public:
 	void check_token_length(char *pos, uint len) const;
 	SubStr token() const;
 	SubStr token(uint start, uint len) const;
-	SubStr raw_token(std::string enclosure) const;
+	Str raw_token(std::string enclosure) const;
 	virtual uint get_line() const;	
 	uint xlat(uint c) const;
 
@@ -66,15 +80,31 @@ public:
 	RegExp * strToRE(SubStr s) const;
 	RegExp * strToCaseInsensitiveRE(SubStr s) const;
 	RegExp * ranToRE(SubStr s) const;
+	RegExp * getAnyRE() const;
 	RegExp * invToRE(SubStr s) const;
 	RegExp * mkDot() const;
 };
 
+inline size_t Scanner::get_pos() const
+{
+	return cur - bot;
+}
+
+inline uint Scanner::get_line() const
+{
+	return cline;
+}
+
 inline uint Scanner::get_cline() const
 {
 	return cline;
 }
 
+inline void Scanner::save_state(ScannerState& state) const
+{
+	state = *this;
+}
+
 inline void Scanner::fatal(const char *msg) const
 {
 	fatal(0, msg);
